// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Patricia K. Fasel (Los Alamos Nat. Lab.), Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 


#include <stdio.h>
#include <strstream.h>
#include <LINK/graph/Attribute.h>
#include <LINK/graph/BinGraph.h>
#include <LINK/graph/Edge.h>
#include <LINK/graph/Vertex.h>


//
// delete the passed vertex from the graph
//
void
MBinGraph::deleteVertex(Vertex* passed_vertex)
{
    MSet<Edge*> inc_edges = passed_vertex->incidentEdges();
    Iterator<Edge*> get_edge(&inc_edges);
    Edge* edge;
    while (get_edge(edge))
	deleteEdge(edge);
    _vertices.remove(passed_vertex);
    delete passed_vertex;
}


//
// construct a new undirected edge from the set of vertices
// add to the graph
//
Edge*
MBinGraph::addEdge(Edge* e)
{
	return MHyperGraph::addEdge(e);
}

ostream& operator<<(ostream&, const Collection<Vertex*>&);

Edge*
MBinGraph::addEdge(Collection<Vertex*>& passed_vertices, String e_name)
{
    if (passed_vertices.size() != 2) {
	error("Binary edge may only contain two vertices");
	return 0;
    }
    Iterator<Vertex*> get_vertex(&passed_vertices);
    Vertex *vertex1, *vertex2;

    get_vertex(vertex1);
    get_vertex(vertex2);
    if (!vertex1 || !vertex2 ||
        (vertex1->owner() != this) || (vertex2->owner() != this)) {
	ostrstream sos;
	sos << "edge " << passed_vertices 
	  	<< "contains vertices not in V(G). Edge not inserted" ;
	char *s = sos.str();
	warning(s);
	delete s;
	return (Edge*) 0;
    }

    Set<Vertex*>* edge_vertices = new Set<Vertex*>;
    Edge* edge = new Edge(this, edge_vertices);

    edge_vertices->insert(vertex1);
    edge_vertices->insert(vertex2);

    vertex1->addEdge(edge);
    vertex2->addEdge(edge);

    _edges.append(edge);

    if (e_name == 0) {
	e_name = makeEdgeName();
    }
    e_name = strcpy(new char[strlen(e_name)+1], e_name);
    setAttribute((GraphObject*) edge, "name", e_name);

    return edge;
}


//
// construct a new directed edge from sequence of vertices
// add to the graph
//
Edge*
MBinGraph::addEdge(Sequence<Vertex*> passed_vertices, String e_name)
{
    if (passed_vertices.size() != 2) {
	error("Binary edge may only contain two vertices");
	return 0;
    }
    Iterator<Vertex*> get_vertex(&passed_vertices);
    Vertex *vertex1, *vertex2;

    get_vertex(vertex1);
    get_vertex(vertex2);

    if (!vertex1 || !vertex2 ||
        (vertex1->owner() != this) || (vertex2->owner() != this)) {
	ostrstream sos;
	sos << "edge " << passed_vertices 
	   	<< "contains vertices not in V(G). Edge not inserted" ;
	char *s = sos.str();
	warning(s);
	delete s;
	return (Edge*) 0;
    }

    Sequence<Vertex*>* edge_vertices = new Sequence<Vertex*>;
    Edge* edge = new Edge(this, edge_vertices);

    edge_vertices->append(vertex1);
    edge_vertices->append(vertex2);

    vertex1->addEdge(edge);
    vertex2->addEdge(edge);

    _edges.append(edge);

    if (e_name == 0)
	e_name = makeEdgeName();
    String new_name = strcpy(new char[strlen(e_name)+1], e_name);
    setAttribute((GraphObject*) edge, "name", new_name);

    return edge;
}


//Edge*
//MBinGraph::addDirectedEdge(Vertex* v1, Vertex* v2)
//{
//    return MHyperGraph::addDirectedEdge(v1, v2);
//}
