// Copyright (C) 1996 DIMACS Center, Rutgers, The State University of New Jersey
// Author(s): Patricia K. Fasel (Los Alamos Nat. Lab.), Jonathan Berry

// This software is copyrighted by the DIMACS Center at Rutgers, The State
// University of New Jersey.  IT IS PROVIDED AS IS, AND THE AUTHORS, DIMACS, AND
// RUTGERS, THE STATE UNIVERSITY OF NEW JERSEY  DISCLAIM
// ALL LIABILITY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
// DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE, ITS DOCUMENTATION, OR ANY
// DERIVATIVES THEREOF, EVEN IF THE AUTHORS HAVE BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.

// THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIM ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, AND NON-INFRINGEMENT.  THIS SOFTWARE
// IS PROVIDED ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAVE
// NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
// MODIFICATIONS.

// The authors hereby grant permission to use, copy, modify, distribute,
// and license this software and its documentation for any purpose, provided
// that existing copyright notices are retained in all copies and that this
// notice is included verbatim in any distributions. No written agreement,
// license, or royalty fee is required for any of the authorized uses.
// Modifications to this software may be copyrighted by their authors
// and need not follow the licensing terms described here, provided that
// the new terms are clearly indicated on the first page of each file where
// they apply.

// Last File Update: 31-Jul-1996
// 

#include <strstream.h>
#include <LINK/graph/DHyperGraph.h>
#include <LINK/graph/Vertex.h>
#include <LINK/graph/Edge.h>
#include <LINK/graph/Attribute.h>
#include <LINK/basic/general.h>
#include <LINK/basic/Sequence.h>


MDHyperGraph::MDHyperGraph() : MHyperGraph()
{
    setAttribute((GraphObject*) this, "direction", (int) Edge::DIRECTED);
}


//
// addEdge method for Set not allowed, turn into a sequence
// addEdge method for Sequence are inherited from HyperGraph
//
Edge*
MDHyperGraph::addEdge(Edge* e)
{
	return MHyperGraph::addEdge(e);
}

Edge*
MDHyperGraph::addEdge(Sequence<Vertex*> passed_vertices, String e_name)
{
	return MHyperGraph::addEdge(passed_vertices, e_name);
}

ostream& operator<<(ostream&, const Collection<Vertex*>&);

Edge*
MDHyperGraph::addEdge(Collection<Vertex*>& passed_vertices, String e_name)
{
    //warning("attempt to construct directed edge with unordered set"
    //	    " -- converting to sequence");
    Sequence<Vertex*>* edge_vertices = new Sequence<Vertex*>;
    Iterator<Vertex*> get_vertex(&passed_vertices);
    Vertex* vertex;
    while (get_vertex(vertex))
	if (vertex && (vertex->owner() == this))		
		edge_vertices->append(vertex);
	else {
		ostrstream sos;
		sos << "edge " << passed_vertices 
	    	<< " contains vertices not in V(G).  Edge not inserted" ;
		char *s = sos.str();
		warning(s);
		delete s;
		delete edge_vertices;
		return (Edge*) 0;
	}
    Edge* edge = new Edge(this, edge_vertices);

    Iterator<Vertex*> get_evertex(edge->vertices());
    while (get_evertex(vertex))
	vertex->addEdge(edge);

    _edges.append(edge);

    if (e_name == 0) {
	e_name = makeEdgeName();
    }
    e_name = strcpy(new char[strlen(e_name)+1], e_name);
    setAttribute((GraphObject*) edge, "name", e_name);

    return edge;
}


//Edge*
//DHyperGraph::addUndirectedEdge(Vertex* v1, Vertex* v2)
//{
//    return addDirectedEdge(v1, v2);
//}
